@@ -10,6 +10,10 @@
 #define CMPLOG_MAP_H 32
 #endif
 
+#define CMPLOG_RTN_LEN 32
+
+#define CMPLOG_MAP_RTN_H ((CMPLOG_MAP_H * sizeof(CmpLogInstruction)) / sizeof(CmpLogRoutine)) 
+
 #define CMPLOG_KIND_INS 0
 #define CMPLOG_KIND_RTN 1
 
@@ -19,24 +23,36 @@ typedef struct CmpLogHeader {
     uint8_t kind;
 } CmpLogHeader;
 
-typedef struct CmpLogOperands {
+typedef struct CmpLogInstruction {
     uint64_t v0;
     uint64_t v1;
-} CmpLogOperands;
+} CmpLogInstruction;
+
+typedef struct CmpLogRoutine {
+    uint8_t v0[CMPLOG_RTN_LEN];
+    uint8_t v1[CMPLOG_RTN_LEN];
+} CmpLogRoutine;
 
 typedef struct CmpLogMap {
   CmpLogHeader headers[CMPLOG_MAP_W];
-  CmpLogOperands operands[CMPLOG_MAP_W][CMPLOG_MAP_H];
+  union {
+      CmpLogInstruction operands[CMPLOG_MAP_W][CMPLOG_MAP_H];
+      CmpLogRoutine routines[CMPLOG_MAP_W][CMPLOG_MAP_RTN_H];
+  } vals;
 } CmpLogMap;
 
 extern CmpLogMap libafl_cmplog_map;
 
 extern uint8_t libafl_cmplog_enabled;
 
-static void __libafl_targets_cmplog(uintptr_t k, uint8_t shape, uint64_t arg1, uint64_t arg2) {
+void __libafl_targets_cmplog_instructions(uintptr_t k, uint8_t shape, uint64_t arg1, uint64_t arg2);
+
+void __libafl_targets_cmplog_routines(uintptr_t k, uint8_t *ptr1, uint8_t *ptr2);
+
+static inline void __libafl_targets_cmplog(uintptr_t k, uint8_t shape, uint64_t arg1, uint64_t arg2) {
 
   if (!libafl_cmplog_enabled) return;
-
+  
   uint16_t hits;
   if (libafl_cmplog_map.headers[k].kind != CMPLOG_KIND_INS) {
     libafl_cmplog_map.headers[k].kind = CMPLOG_KIND_INS;
@@ -51,8 +67,8 @@ static void __libafl_targets_cmplog(uintptr_t k, uint8_t shape, uint64_t arg1, u
   }
 
   hits &= CMPLOG_MAP_H - 1;
-  libafl_cmplog_map.operands[k][hits].v0 = arg1;
-  libafl_cmplog_map.operands[k][hits].v1 = arg2;
+  libafl_cmplog_map.vals.operands[k][hits].v0 = arg1;
+  libafl_cmplog_map.vals.operands[k][hits].v1 = arg2;
   
 }
 
